unit dbase;

INTERFACE

{uses Crt;}

function OpenDBFData(dbffilename :string):byte;
function OpenDBFMemo:byte;

function ReadDBFRecord (I : Longint):byte;
function ReadDBFMemo(BlockNumber : integer):byte;

Procedure WriteDBFRecord;
Procedure WriteDBFMemo(var BlockNumberString : string);
Procedure CloseDBFData;
Procedure CloseDBFMemo;

const
   DBFMaxRecordLength = 4096;
   DBFMemoBlockLength = 512;
   DBFMaxMemoLength   = 4096;

      {Error handling:}
      ErrOK              = 0;
      ErrFileNotFound    = 101;
      ErrReading         = 102;
      ErrWriting         = 103;
      ErrBlockID         = 104;
      ErrFileID          = 105;
      ErrPosition        = 106;
      ErrDataSize        = 107;

type
  DBFHeaderRec = Record
                 HeadType     : byte;
                 Year         : byte;
                 Month        : byte;
                 Day          : byte;
                 RecordCount  : longint;
                 HeaderLength : integer;
                 RecordSize   : integer;
                 Garbage      : array[1..20] of byte;
               end;

type
  DBFFieldRec = Record
                FieldName     : array[1..11] of char;
                FieldType     : char;
                Spare1,
                Spare2        : integer;
                Width         : byte;
                Dec           : byte;
                WorkSpace     : array[1..14] of byte;
              end;

var
  DBFFileName          : string;
  DBFDataFile          : File;
  DBFDataFileAvailable : boolean;
  DBFBuffer            : array [1..DBFMaxRecordLength] of char;
  DBFHeading           : DBFHeaderRec;
  DBFField             : DBFFieldRec;
  DBFFieldCount        : integer;
  DBFFieldContent      : array [1..128] of string;
  DBFNames             : array [1..128] of string[10];
  DBFLengths           : array [1..128] of byte;
  DBFTypes             : array [1..128] of char;
  DBFDecimals          : array [1..128] of byte;
  DBFContentStart      : array [1..128] of integer;
  DBFMemoFile          : File;
  DBFMemoFileAvailable : boolean;
  DBFMemoBuffer        : Array [1..DBFMemoBlockLength] of byte;
  DBFMemo              : Array [1..DBFMaxMemoLength] of char;
  DBFMemoLength        : integer;
  DBFMemoEnd           : boolean;
  DBFMemoBlock         : integer;
  DBFDeleteField       : char;
  DBFFieldStart        : integer;
  DBFRecordNumber      : longint;

IMPLEMENTATION

function ReadDBFHeader:byte;{***************************************************}

var
  RecordsRead : integer;

begin
  BlockRead (DBFDataFile, DBFHeading, SizeOf(DBFHeading), RecordsRead);
  if SizeOf(DBFHeading)<> RecordsRead then ReadDBFHeader :=ErrReading
                                      else ReadDBFHeader :=ErrOK;
end;

Procedure ProcessField (F : DBFFieldRec;I : integer);{**********************}

var
  J : integer;

begin
  with F do begin
    DBFNames [I] := '';
    J := 1;
    while (J<11) and (FieldName[J] <> #0) do begin
      DBFNames[I] := DBFNames[I] + FieldName [J];
      J := J + 1;
    end;
    DBFLengths [I]      := Width;
    DBFTypes [I]        := FieldType;
    DBFDecimals [I]     := Dec;
    DBFContentStart [I] := DBFFieldStart;
    DBFFieldStart       := DBFFieldStart + Width;
  end;
end;

function ReadFields:byte;{******************************************************}

var
  I           : integer;
  RecordsRead : integer;

begin
  Seek(DBFDataFile,32);
  I := 1;
  DBFFieldStart := 2;
  DBFField.FieldName[1] := ' ';
  while (DBFField.FieldName[1] <> #13) do begin
    BlockRead(DBFDataFile,DBFField.FieldName[1],1);
    if (DBFField.FieldName[1] <> #13) then begin
      BlockRead(DBFDataFile, DBFField.FieldName[2],SizeOf(DBFField) - 1,
                                                             RecordsRead);
      if (SizeOf(DBFField)-1)<>RecordsRead then begin;ReadFields:=ErrReading;exit;end
                                           else ReadFields:=ErrOK;
      ProcessField (DBFField, I);
      I := I + 1;
    end;
  end;
  DBFFieldCount := I - 1;
end;

function OpenDBFData(dbffilename :string):byte;{********************************}

begin
  DBFDataFileAvailable := false;
  if Pos('.',DBFFileName)>0 then Assign(DBFDataFile, DBFFileName)
                            else Assign(DBFDataFile, DBFFileName+'.DBF');
  {$I-} Reset(DBFDataFile,1); {$I+}
  If IOResult<>0 then begin;
                        OpenDBFData:=ErrFileNotFound;
                        exit;
                      end;
  DBFDataFileAvailable := true;
  OpenDBFData:=ErrOK;
  Seek(DBFDataFile,0);
  if ReadDBFHeader<>ErrOK then begin;
                        OpenDBFData:=ErrReading;
                        exit;
                      end;
  if ReadFields<>ErrOK then begin;
                        OpenDBFData:=ErrReading;
                        exit;
                      end;
  OpenDBFData:=ErrOK;
end;

Procedure CloseDBFData;{****************************************************}

begin
  if DBFDataFileAvailable then Close(DBFDataFile);
end;

function OpenDBFMemo:byte;{*****************************************************}

begin
  DBFMemoFileAvailable := false;
{  Assign(DBFMemoFile, DBFFileName+'.DBT');}
  if Pos('.',DBFFileName)>0
     then Assign(DBFDataFile, copy(DBFFileName,1,Pos('.',DBFFileName)-1)+'.DBT')
     else Assign(DBFDataFile, DBFFileName+'.DBT');
  {$I-} Reset(DBFMemoFile,1);{$I+}
  If IOResult<>0 then
  begin;
    if Pos('.',DBFFileName)>0
       then Assign(DBFDataFile, copy(DBFFileName,1,Pos('.',DBFFileName)-1)+'.FPT')
       else Assign(DBFDataFile, DBFFileName+'.FPT');
    {$I-} Reset(DBFMemoFile,1);{$I+}
    If IOResult<>0 then begin;
                        OpenDBFMemo:=ErrFileNotFound;
                        exit;
                      end;
  end;
  DBFMemoFileAvailable := true;
  Seek(DBFMemoFile,0);
  OpenDBFMemo:=ErrOK;
end;

Procedure CloseDBFMemo;{****************************************************}

begin
  If DBFMemoFileAvailable then close(DBFMemoFile);
end;

Procedure GetDBFFields;{****************************************************}

var
  I        :byte;
  J        :integer;
  Response :string;

begin
  DBFDeleteField := DBFBuffer[1];
  For I:=1 to DBFFieldCount do begin
    DBFFieldContent[I] := '';
    For J := DBFContentStart[I] to DBFContentStart [I] + DBFLengths[I] -1 do
       DBFFieldContent[I] := DBFFieldContent[I] + DBFBuffer[J];
    For J := 1 to DBFLengths[I] do
      if DBFFieldContent[J]=#0 then DBFFieldContent[J]:=#32;
  end;
end;

function ReadDBFRecord (I : Longint):byte;{*************************************}

var
  RecordsRead : integer;

begin
  Seek(DBFDataFile,DBFHeading.HeaderLength +DBFHeading.RecordSize *(I - 1));
  BlockRead (DBFDataFile, DBFBuffer, DBFHeading.RecordSize, RecordsRead);
  if SizeOf(DBFHeading.RecordSize)<> RecordsRead then ReadDBFRecord :=ErrReading
                                      else ReadDBFRecord :=ErrOK;
  GetDBFFields;
end;

function ReadDBFMemo(BlockNumber : integer):byte;{******************************}

var
  I           :integer;
  RecordsRead :word;

begin
  DBFMemoLength := 0;
  DBFMemoEnd := false;
  If not DBFMemoFileAvailable then begin
    DBFMemoEnd := true;
    exit;
  end;
  FillChar(DBFMemo[1],DBFMaxMemoLength,#0);
  Seek(DBFMemoFile,BlockNumber*DBFMemoBlockLength);
  repeat
    BlockRead(DBFMemoFile,DBFMemoBuffer,DBFMemoBlockLength,RecordsRead);
    if DBFMemoBlockLength<>RecordsRead then begin;ReadDBFMemo:=ErrReading;exit;end
                                         else ReadDBFMemo:=ErrOK;
    For I := 1 to RecordsRead  do begin
      DBFMemoLength := DBFMemoLength + 1;
      DBFMemo[DBFMemoLength] := chr(DBFMemoBuffer[I] and $7F);
      If (DBFMemoBuffer[I] = $1A) or (DBFMemoBuffer[I] = $00) then begin
        DBFMemoEnd := true;
        DBFMemoLength := DBFMemoLength - 1;
        exit;
      end;
    end;
  until DBFMemoEnd;
end;

Procedure WriteDBFMemo{(var BlockNumberString : string)};{******************}

var
  K          : integer;
  ReturnCode : integer;

begin
  Val(BlockNumberString,DBFMemoBlock,ReturnCode);
  If ReturnCode>0 then DBFMemoBlock := 0;
  If DBFMemoBlock>0 then begin
    Writeln;
    ReadDBFMemo(DBFMemoBlock);
    If DBFMemoLength=0 then exit;
    For K := 1 to DBFMemoLength do Write(DBFMemo[K]);
    WriteLn;
  end;
end;

Procedure WriteDBFRecord;{**************************************************}

var
  J : byte;

begin
  For J := 1 to DBFFieldCount do begin
{    textcolor(7);}
    Write(DBFNames[J]);
{    GoToXY(12,J);
    textcolor(3);}
    WriteLn(DBFFieldContent[J]);
    if DBFTypes[J]='M' then WriteDBFMemo(DBFFieldContent[J]);
  end;
end;

begin
end.


