{$N+,E+,F-}
{****************************************************************************
 *           } Unit UCalcul;{ (c) 1999 Michal Cihar                         *
 ****************************************************************************
 ****************************************************************************
 * Pravidla pro vzorce:                                                     *
 *  - pri nasobeni je mozno vynechat znamenko (5x=5*x), ale pokud je ve     *
 *    vyrazu x5 tak to znamena promena s nazvem "X5"                        *
 *  - zavorky: - pokud je ve vzorci vice zacatku nez koncu zavorek          *
 *               procedura doplni potrebny pocet koncu zavorek na konec     *
 *               vyrazu                                                     *
 *             - pokud je ve vzorci vice koncu nez zacatku zavorek jedna se *
 *               o chybu                                                    *
 *  - priklady:                                                             *
 *   vyraz    je ekvivaletni s   vyrazem                                    *
 *   5x                          5*x                                        *
 *   5(x+1                       5*(x+1)                                    *
 ****************************************************************************
 * Popis:                                                                   *
 *  Unit Calcul obsahuje funkci pro rekurzivni vyhodnocovani vyrazu         *
 *  zadaneho behem spusteni programu.                                       *
 *                                                                          *
 *  Podporovany jsou operatory (+ - * / \ % ~ ^ ! =), zavorky, funkce,      *
 *  konstanty a promenne. Vetsina funkci je pocitana standartnimi funkcemi  *
 *  obsazenymi v Turbo (Borland) Pascalu krome nekterych funkci, ktere      *
 *  jsou podporovany rozhranim Borlandu ke koprocesoru nebo k emulatoru.    *
 *  Cisla (vcetne promennych) jsou uchovavana v typu extended (rozsah       *
 *  3.4e-4932..1.1e4932 plus znamenko, platnych cislic 19-20, bytu: 10).    *
 *  Pri vypoctech se provadi kontrola ErrOverflow, podteceni kontrolovano     *
 *  neni. Promenne muzou byt ulozeny do souboru na disku.                   *
 *                                                                          *
 *  Vyraz je vyhodnocovan zleva doprava a operatory (funkce) jsou           *
 *  vyhodnocovany v poradi podle priorit:                                   *
 *        1. =             (prirazeni)                                      *
 *        2. funkce                                                         *
 *           5pi,~         (nasobek promenne/konstanty,exponent)            *
 *        3. (,)           (zavorky)                                        *
 *        4. !             (faktorial)                                      *
 *        5. +,-           (znamenko)                                       *
 *        6. ^             (mocnina)                                        *
 *        7. *,/,%,MOD,DIV (nasobeni,deleni,celociselne deleni)             *
 *        8. +,-           (scitani,odcitani)                               *
 ****************************************************************************
 * Historie:                                                                *
 *  #=informace, -=odstraneno, +=pridano, !=datum, %=zmena                  *
 *  Verze : Zmeny                                                           *
 *    0.1 : # prvni snad funkcni verze                                      *
 *          ! 21.3.1998                                                     *
 *    0.11: # prvni opravdu funkcni verze  ;-)                              *
 *          ! 28.3.1998                                                     *
 *          - odstraneny chyby: ve funkcich, vyhodnocovani, ...             *
 *          - odstranena funkce ZRUS (presunuta do MISO Calc)               *
 *    0.2 : # zmena ve zpracovani nekterych funkci                          *
 *          # zatim to vypada, ze to opravdu funguje                        *
 *          ! 25.11.1998                                                    *
 *          % rozdeleno do vice unit                                        *
 *          + parametry se daji zadavat bez (): sin pi                      *
 *          + vyhodnoceni vyrazu 5(... jako 5*(...                          *
 *          - odstraneny chyby v zaokrouhleni a mocninach                   *
 *          + Result minule funkce je uchovavan v konstante ANS           *
 *          + pokud vyraz zacina operaci je pred tuto operaci dosazen       *
 *            Result minuleho vypoctu *12 je ANS*12                       *
 *    0.5 : # Zmena vyhodnocovani chyb                                      *
 *          ! 27.11.1998                                                    *
 *          % pokud vyraz zacina operaci +/- neni dosazeno ANS (viz 0.2)    *
 *          % pokud cislo obsahuje vice desetinnych tecek pouzije se prvni  *
 *          + nyni se da zjistit typ chyby                                  *
 *          + pridan operator \ - zbytek po celociselnem deleni             *
 *          - zrusen typ druh operace a souvisejici procedury               *
 *    0.6 : # Pridany funkce                                                *
 *          !  2.12.1998                                                    *
 *          % opravena chyba: cislo staci zadat .9 neni potreba 0.9         *
 *          % FunctionData je promenna                                        *
 *          % pokud vyraz zacina operaci + je dosazeno ANS (viz 0.2 a 0.5)  *
 *          + pridany hyperbolicke a hyperbolometricke funkce               *
 *          + pridan faktorial                                              *
 *          + pridany kombinace a variace                                   *
 *          + v konstantach pribyly predpony jednotek (Exa-yokto)           *
 *    0.7 : # Opraveny chyby                                                *
 *          ! 10.12.1998                                                    *
 *          % opravena chyba: 1neco neni run time error                     *
 *          % opravena chyba: sin neni run time error                       *
 *    1.0: # Opraveny chyby                                                *
 *          ! 16.12.1998                                                    *
 *          % ANS se nepridava v teto unit, ale v MISO Calc                 *
 *          % opravena chyba / neni run time error                          *
 *          % u periodickych fuknci se prepocitava parametr do zakl. periody*
 *          + ruzne jednotky uhlu (deg, rad, grad)                          *
 *          + konverze jednotek uhlu                                        *
 *          - odstranena spousta chyb pri neocekavanem konci vyrazu         *
 *    1.0 : # Opraveny chyby                                                *
 *          # Prvni uvolnena verze                                          *
 *          ! 5.1.1999                                                      *
 *          % odstaneny ruzne malickate chybicky :-)                        *
 *          + pridana kontrola ErrOverflow (snad funkcni)                     *
 *          + pribyly funkce MAX a MIN                                      *
 *    1.1 : # Opraveny chyby, pridany funkce                                *
 *          ! 9.1.1999                                                      *
 *          % odstanena chyba v 2. mocnine/odmocnine - kontrola < 0         *
 *          + pribyly funkce MAXA, MINA, CU, CUR, INV, AVG, GAV a HAV       *
 *    1.5 : # Pridana prace s ciselnymi soustavami                          *
 *          ! ??.2.1999 (bude)                                              *
 *          + pri cteni cisla: $x znaci sesnactkove cislo                   *
 *                             #h/#d/#o/#b x znaci cislo v soustave hex,    *
 *                                         dec, oct, bin                    *
 *                             #n, x/#n x cislo v soustave o zakladu n      *
 *                                        n je od 2 do 36                   *
 *          - odstranena chyba pri vyrazu typu (1+2))-1 (rozsirena kontrola *
 *            poctu zacatku a koncu zavorek)                                *
 *          + pridany jmena pro funkce tangens - nyni funguje *tan* i *tg*  *
 *            (* znamena libovolny doplnek k jmenu napr: argtgh)            *
 *          + pridana funkce CEIL                                           *
 *          + nyni je mozno pouzit neomezeny pocet znamenek napr: -- -- --1 *
 *          % zmena formatu souboru - MISO File Format - podpora ruznych    *
 *            bloku dat, rozsiritelnost, ukladana jen potrebna data         *
 *          % operace se soubory pres unit UFiles - nacteni bufferu a prace *
 *            s nim jako se souborem                                        *
 *          + pridany operatory MOD a DIV                                   *
 *          % od teto verze "1+1 1" chyba, vyraz je vyhodnocovan az do      *
 *            posledniho znaku, nebo do ';'                                 *
 *          + pridano numericke derivovani - DERIVATE                       *
 *          + pridana druha derivace - DERIVATE2                            *
 *          + pridano reseni rovnic - SOLVE a SOLVEG                        *
 *          + pridana sumace - SUM                                          *
 *          + pridano numericke integrovani - INTG a INTGS                  *
 *                                                                          *
 *                                                                          *
 ****************************************************************************}

{****************************************************************************
 *                             }Interface{                                  *
 ****************************************************************************}
uses Umath,Umemory,Udata,UBase,strfunc;
const UnitID='UCalcul';
      UnitCopyright='1999 Michal Cihar';
      UnitVersion='1.57';
      UnitRelease='25.2.1999';

type tWritelnProc=procedure(const s:string);
     tPauseProc=procedure;
procedure UCWriteln(const s:string);
procedure UCPause;
const SumTableMsg:string=' Count          Value                  Cumulative sum';
      SumTablePause:longint=24;
      SumTableWriteln:tWritelnProc=UCwriteln;
      SumTablePauseProc:tPauseProc=UCPause;

function Evaluate(What : tString;var IsValid:Boolean) : tRealNumber;
function Sum(const What,Variable:tString;const FromN,ToN:tRealNumber;var ErrorAt:byte):tRealNumber;
function SumTable(const What,Variable:tString;const FromN,ToN:tRealNumber;var ErrorAt:byte):tRealNumber;
function Intg(const What,Variable:tString;const FromN,ToN:tRealNumber;var ErrorAt:byte):tRealNumber;
function IntgS(const What,Variable:tString;const FromN,ToN:tRealNumber;var ErrorAt:byte):tRealNumber;
function Derivate(const What,Variable:tString;const where:tRealNumber;var ErrorAt:byte):tRealNumber;
function Derivate2(const What,Variable:tString;const where:tRealNumber;var ErrorAt:byte):tRealNumber;
function Solver(const What,Variable:tString;var ErrorAt:byte):tRealNumber;
function Solverg(const What,Variable:tString;var guess:tRealNumber;var ErrorAt:byte):tRealNumber;

{****************************************************************************
 *                           }implementation{                               *
 ****************************************************************************}
{ted definuju vsechny forward procedury a funkce}
procedure Priority2(var Result : tRealNumber;var Express:tExpression) ;    forward;
procedure Priority5(var Result : tRealNumber;var Express:tExpression) ;    forward;
procedure Priority6(var Result : tRealNumber;var Express:tExpression) ;    forward;
procedure EvalFunction(var Result : tRealNumber;var Express:tExpression); forward;

procedure UCWriteln(const s:string);
begin;
WriteLn(s);
end;

procedure UCPause;
begin;
write('Press ENTER to continue...');
readln;
end;
{****************************************************************************
 *                             PrepareError                                 *
 ****************************************************************************
 Nastavi globalni promenne na hodnotu aktualni chyby}
procedure PrepareError(const Number,Where:byte;const Info:string);
begin
  ErrorNumber:=Number;
  ErrorPosition:=Where;
  ErrorString:=Info;
end; {PrepareError}

{****************************************************************************
 *                             IgnoreSpaces                                *
 ****************************************************************************
 Preskoci v promenne Vyraz mezeru od pozice Express.Position a nastavi Express.Position na tuto
 pozici}
procedure IgnoreSpaces(var Express:tExpression);
begin
     While (length(Express.Expression)>=Express.Position)and
           (Express.Expression [Express.Position] = ' ') do Inc(Express.Position);
end; {IgnoreSpaces}

{****************************************************************************
 *                             ReadSubExpression                                  *
 ****************************************************************************
 Cte z promenne Vyraz SubExpression oddeleny znakem z promenne Divisors}
function ReadSubExpression(var Express:tExpression) : string;
var
   Reading : string;
begin
     Reading := '';
     While (Express.Position <= Length(Express.Expression)) and
           not(Express.Expression [Express.Position] in Divisors) do
     begin
          Reading := Reading + UpCase(Express.Expression[Express.Position]);
          Inc(Express.Position);
     end;
     ReadSubExpression := Reading;
end; {ReadSubExpression}

{****************************************************************************
 *                             ReadNumBase                                  *
 ****************************************************************************
 Cte cislo nebo nasobek konstanty nebo funkce z promenne Express.Expression}
function ReadNumBase(var Express:tExpression) : tRealNumber;
var
   Value1: tRealNumber;
   Code   : Integer;
   Base:byte;
   b:boolean;
   SubExpression : string;
   NotNumber:string;
   op:char;
   TempPosition:byte;
begin;
  TempPosition:=Express.Position;
  if Express.Expression[Express.Position]='$' then
  begin;
  inc(Express.Position);
  IgnoreSpaces(Express);
  SubExpression := ReadSubExpression(Express);
  if ConvertTo10(SubExpression,16,Value1)<> 0 then
  begin;
     PrepareError(ErrBadBase,TempPosition,SubExpression);
     exit;
  end;
  end else
  if Express.Expression[Express.Position]='#' then
  begin;
     case Express.Expression[Express.Position+1] of
     '0'..'9':
     begin;
       inc(Express.Position);
       SubExpression := ReadSubExpression(Express);
       val(SubExpression,Base,Code);
       if Code<>0 then Base:=10;
       if (Base<2)or(Base>36) then
          begin;
            PrepareError(ErrBaseOutOfRange,TempPosition,'');
            exit;
          end;
     end;
     'h','H':begin;Base:=16;Express.Position:=Express.Position+2;end;
     'd','D':begin;Base:=10;Express.Position:=Express.Position+2;end;
     'o','O':begin;Base:=8;Express.Position:=Express.Position+2;end;
     'b','B':begin;Base:=2;Express.Position:=Express.Position+2;end;
     end;
     IgnoreSpaces(Express);
     if Express.Expression[Express.Position]=',' then inc(Express.Position);
     IgnoreSpaces(Express);
     SubExpression := ReadSubExpression(Express);
     if ConvertTo10(SubExpression,Base,Value1)<> 0 then
        PrepareError(ErrBadBase,TempPosition,SubExpression);
  end;
  ReadNumBase:=Value1;
end;

{****************************************************************************
 *                             ReadNumber                                   *
 ****************************************************************************
 Cte cislo nebo nasobek konstanty nebo funkce z promenne Express.Expression}
function ReadNumber(var Express:tExpression) : tRealNumber;
var
   Value1,Value2,znamenko: tRealNumber;
   Code,Code2   : Integer;
   b:boolean;
   SubExpression : string;
   op:char;
   TempPosition:byte;
begin
  IgnoreSpaces(Express);
  TempPosition:=Express.Position;
  SubExpression := ReadSubExpression(Express);
  if SubExpression[1] = '.' then SubExpression := '0' + SubExpression else
  if SubExpression[1] = '~' then SubExpression := '1' + SubExpression;
  if upcase(SubExpression[1]) in ['A'..'Z','_'] then
  begin;
     EvalFunction(Value1,Express);
  end
  else {neni pismeno ani _}
  begin;
     Code:=Str2Number(SubExpression,Value1);
     if Code <> 0 then
     if SubExpression[length(SubExpression)]='~' then
     begin;
        IgnoreSpaces(Express);
        if Express.Expression[Express.Position] in ['+','-'] then
        begin;
           op:=Express.Expression[Express.Position];
           Code:=Str2Number(copy(SubExpression,1,Code-1),Value1);
           inc(Express.Position);
           IgnoreSpaces(Express);
           TempPosition:=Express.Position;
           SubExpression:=ReadSubExpression(Express);
           Code:=Str2Number(SubExpression,Value2);
           if Code = 0 then
           begin;
              case op of '+':begin;
                              if testp10(Value2)and testnas(Value1,power10(Value2)) then
                              Value1:=Value1*power10(Value2)
                              else PrepareError(ErrOverflow,TempPosition,'');
                             end;
                         '-':Value1:=Value1*power10(-(Value2));
                         end;
           end else
           begin;
              Express.Position:=TempPosition+Code-1;
              Value1 := 0;
              PrepareError(ErrBadExponent,Express.Position,SubExpression);
           end;
        end
        else {neni +/-}
        begin;
           Express.Position:=TempPosition+Code-1;
           Value1 := 0;
           PrepareError(ErrNoExponent,Express.Position,'');
        end;
     end else {posledni neni ~}
     if pos('~',SubExpression)>0 then
     begin;
           Code2:=Str2Number(copy(SubExpression,Code+1,length(SubExpression)-Code),Value2);
           Code:=Str2Number(copy(SubExpression,1,Code-1),Value1);
           if Code2 = 0 then
                             begin;
                              if testp10(Value2)and testnas(Value1,power10(Value2)) then
                              Value1:=Value1*power10(Value2)
                              else PrepareError(ErrOverflow,TempPosition,'');
                             end
           else
           begin;
              Express.Position:=TempPosition+Code+Code2-1;
              Value1 := 0;
              PrepareError(ErrBadExponent,Express.Position,SubExpression);
           end;
     end
     else {neni ~}
     if upcase(SubExpression[Code]) in ['A'..'Z','_'] then
        begin;
           Str2Number(copy(SubExpression,1,Code-1),Value1);
           Express.Position:=TempPosition+Code-1;
           EvalFunction(Value2,express);
           if ErrorNumber = 0 then
           Value1:=Value1*Value2;
     end
     else {neni pismeno ani _}
     begin;
        Express.Position:=TempPosition+Code-1;
        Value1 := 0;
        PrepareError(ErrNotNumber,Express.Position,SubExpression);
     end;
  end;
  ReadNumber := {znamenko*}Value1;
end; {ReadNumber}


{****************************************************************************
 *                             ZjistiParamer                                *
 ****************************************************************************
 Cte prametr z promenne Express.Expression a Express.Position Express.Position }
procedure GetSingleParameter(var Result : tRealNumber;var Express:tExpression);
begin
        IgnoreSpaces(Express);
        if (Express.Expression [Express.Position] <> '(') then
           begin;
                     Priority5(Result,Express);
                     IgnoreSpaces(Express);
           end
           else begin
                     Priority6(Result,Express);
                     IgnoreSpaces(Express);
           end;
end; {GetSingleParameter}

{****************************************************************************
 *                            ZjistiParamer0                                *
 ****************************************************************************
 Cte prametr z promenne Express.Expression a Express.Position Express.Position }
procedure GetCurrentParameter(var Result : tRealNumber;var Express:tExpression);
begin
        IgnoreSpaces(Express);
        if (Express.Expression [Express.Position] = ',') then
        begin;
        inc(Express.Position);
        IgnoreSpaces(Express);
        end;
        Priority2(Result,Express);
        IgnoreSpaces(Express); {popojdem k ,}
        if ErrorNumber=ErrOK then
        if Express.Expression[Express.Position] <> ',' then
        PrepareError(ErrNoComma,Express.Position,'');
end; {GetCurrentParameter}

{****************************************************************************
 *                             GetTwoParameters                             *
 ****************************************************************************
 Cte 2 prametry z promenne Express.Expression a Express.Position Express.Position oddelene carkou}
procedure GetTwoParameters(var Vysledek1,Vysledek2 : tRealNumber;var Express:tExpression);
begin
        ReadSubExpression(Express);
        IgnoreSpaces(Express);
        if (Express.Expression [Express.Position] <> '(') then
                        PrepareError(ErrNoBracketStart,Express.Position,'')
           else begin
                     Inc(Express.Position);
                     Priority2(Vysledek1,Express);
                     IgnoreSpaces(Express); {popojdem k ,}
                     if Express.Expression[Express.Position] <> ',' then
                        PrepareError(ErrNoComma,Express.Position,'') else
                     begin;;
                     Inc(Express.Position);
                     IgnoreSpaces(Express);
                     Priority2(Vysledek2,Express);
                     IgnoreSpaces(Express); {popojdem k )}
                     if Express.Expression[Express.Position] <> ')' then
                        PrepareError(ErrNoBracketEnd,Express.Position,'') else
                     begin;
                     Inc(Express.Position);
                     IgnoreSpaces(Express);
                     end;
                     end;
           end;
end; {GetTwoParameters}

function Sum(const What,Variable:tString;const FromN,ToN:tRealNumber;var ErrorAt:byte):tRealNumber;
var b,WasParameter:boolean;
    Result1,Result2,Current,PrmValue:tRealNumber;
begin
     Result2:=0.0;
     WasParameter:=IsParameter(Variable);
     if WasParameter then PrmValue:=GetParameter(Variable,b);
     if ErrorNumber=ErrOK then
     begin;
     if FromN<ToN then
     begin;
     Current:=realtrunc(FromN);
     while (ErrorNumber=ErrOK)and(Current<=ToN) do
     begin;
     IntoParameters(Variable,Current);
     Result1:=Evaluate(What,b);
     if testscit(Result1,Result2) then Result2:=Result2+Result1
                              else begin;PrepareError(ErrOverflow,1,'');break;end;
     Current:=Current+1.0;
     end;
     end else begin;PrepareError(ErrBadSum,1,'');Sum := 0;end;
     end else Sum := 0;
     Sum := Result2;
     if WasParameter then IntoParameters(Variable,PrmValue)
                     else DeleteParameter(Variable);
     ErrorAt:=ErrorPosition;
end; {Sum}

function SumTable(const What,Variable:tString;const FromN,ToN:tRealNumber;var ErrorAt:byte):tRealNumber;
var b,WasParameter:boolean;
    Result1,Result2,Current,PrmValue:tRealNumber;
    LinesDone:longint;
begin
     WasParameter:=IsParameter(Variable);
     if WasParameter then PrmValue:=GetParameter(Variable,b);
     SumTablewriteln(SumTableMsg);
     Result2:=0.0;
     if ErrorNumber=ErrOK then
     begin;
     if FromN<ToN then
     begin;
     Current:=realtrunc(FromN);
     LinesDone:=0;
     while (ErrorNumber=ErrOK)and(Current<=ToN) do
     begin;
     IntoParameters(Variable,Current);
     Result1:=Evaluate(What,b);
     if testscit(Result1,Result2) then Result2:=Result2+Result1
                              else begin;PrepareError(ErrOverflow,1,'');break;end;
     if LinesDone>=SumTablePause then begin;
                                        SumTablePauseProc;
                                        LinesDone:=0;
                                        SumTablewriteln(SumTableMsg);
                                      end;
     inc(LinesDone);
     SumTablewriteln(' '+Number2String(Current)+ReturnNchars(' ',15-length(Number2String(Current)))+
             Number2String(Result1)+ReturnNchars(' ',23-length(Number2String(Result1)))+
             Number2String(Result2)+ReturnNchars(' ',23-length(Number2String(Result2))));
     Current:=Current+1.0;
     end;
     end else begin;PrepareError(ErrBadSum,1,'');SumTable := 0;end;
     end else SumTable := 0;
     SumTable := Result2;
     if WasParameter then IntoParameters(Variable,PrmValue)
                     else DeleteParameter(Variable);
     ErrorAt:=ErrorPosition;
end; {SumTable}

function Intg(const What,Variable:tString;const FromN,ToN:tRealNumber;var ErrorAt:byte):tRealNumber;
var b,WasParameter:boolean;
    DeltaUsed,Result1,Result2,Result,Current,PrmValue:tRealNumber;
begin
     Result:=0.0;
     WasParameter:=IsParameter(Variable);
     if WasParameter then PrmValue:=GetParameter(Variable,b);
     if ErrorNumber=ErrOK then
     begin;
     if FromN<ToN then
     begin;
     DeltaUsed:=(ToN-FromN)/IntgDivs;
     Current:=FromN;
     IntoParameters(Variable,Current);
     Result2:=Evaluate(What,b);
     while (ErrorNumber=ErrOK)and(Current<ToN) do
     begin;
     Current:=Current+DeltaUsed;
     IntoParameters(Variable,Current);
     Result1:=Evaluate(What,b);
     if not(testscit(Result1,Result2)) then begin;PrepareError(ErrOverflow,1,'');break;end;
     if not(testnas(Result1+Result2,DeltaUsed/2)) then begin;PrepareError(ErrOverflow,1,'');break;end;
     Result:=Result+((Result1+Result2)*DeltaUsed/2);
     Result2:=Result1;
     end;
     end else begin;PrepareError(ErrBadIntg,1,'');Intg := 0;end;
     end else Intg := 0;
     Intg := Result;
     if WasParameter then IntoParameters(Variable,PrmValue)
                     else DeleteParameter(Variable);
     ErrorAt:=ErrorPosition;
end; {Intg}

function IntgS(const What,Variable:tString;const FromN,ToN:tRealNumber;var ErrorAt:byte):tRealNumber;
var b,WasParameter:boolean;
    DeltaUsed,Result1,Result2,Result,Current,PrmValue:tRealNumber;
begin
     Result:=0.0;
     WasParameter:=IsParameter(Variable);
     if WasParameter then PrmValue:=GetParameter(Variable,b);
     if ErrorNumber=ErrOK then
     begin;
     if FromN<ToN then
     begin;
     DeltaUsed:=(ToN-FromN)/IntgDivs;
     Current:=FromN+(DeltaUsed/2);
     while (ErrorNumber=ErrOK)and(Current<ToN) do
     begin;
     IntoParameters(Variable,Current);
     Result1:=Evaluate(What,b);
     if not(testnas(Result1,DeltaUsed)) then begin;PrepareError(ErrOverflow,1,'');break;end;
     Result:=Result+(Result1*DeltaUsed);
     Current:=Current+DeltaUsed;
     end;
     end else begin;PrepareError(ErrBadIntg,1,'');IntgS := 0;end;
     end else IntgS := 0;
     IntgS := Result;
     if WasParameter then IntoParameters(Variable,PrmValue)
                     else DeleteParameter(Variable);
     ErrorAt:=ErrorPosition;
end; {IntgS}

function Derivate(const What,Variable:tString;const where:tRealNumber;var ErrorAt:byte):tRealNumber;
var b,WasParameter:boolean;
    Result1,Result2,PrmValue:tRealNumber;
begin
     if ErrorNumber=ErrOK then
     begin;
     WasParameter:=IsParameter(Variable);
     if WasParameter then PrmValue:=GetParameter(Variable,b);
     IntoParameters(Variable,where-delta);
     Result1:=Evaluate(What,b);
     if ErrorNumber=ErrOK then
     begin;
     IntoParameters(Variable,where+delta);
     Result2:=Evaluate(What,b);
     if ErrorNumber=ErrOK then
     begin;
     derivate:=(Result2-Result1)/(2*delta);
     end else derivate := 0;
     end else derivate := 0;
     if WasParameter then IntoParameters(Variable,PrmValue)
                     else DeleteParameter(Variable);
     end else derivate := 0;
     ErrorAt:=ErrorPosition;
end; {Derivate}

function Derivate2(const What,Variable:tString;const where:tRealNumber;var ErrorAt:byte):tRealNumber;
var b,WasParameter:boolean;
    Result1,Result2,Result3,Result4,Result5,PrmValue:tRealNumber;
begin
     if ErrorNumber=ErrOK then
     begin;
     WasParameter:=IsParameter(Variable);
     if WasParameter then PrmValue:=GetParameter(Variable,b);
{     DeleteAllParameters;}
     IntoParameters(Variable,where-(2*delta));
     Result1:=Evaluate(What,b);
     if ErrorNumber=ErrOK then
     begin;
     IntoParameters(Variable,where-delta);
     Result2:=Evaluate(What,b);
     if ErrorNumber=ErrOK then
     begin;
     IntoParameters(Variable,where);
     Result3:=Evaluate(What,b);
     if ErrorNumber=ErrOK then
     begin;
     IntoParameters(Variable,where+delta);
     Result4:=Evaluate(What,b);
     if ErrorNumber=ErrOK then
     begin;
     IntoParameters(Variable,where+(2*delta));
     Result5:=Evaluate(What,b);
     if ErrorNumber=ErrOK then
     begin;
     derivate2:=(-Result1+(16*Result2)-(30*Result3)+(16*Result4)-Result5)/(16*delta*delta);
     end else derivate2 := 0;
     end else derivate2 := 0;
     end else derivate2 := 0;
     end else derivate2 := 0;
     end else derivate2 := 0;
     if WasParameter then IntoParameters(Variable,PrmValue)
                     else DeleteParameter(Variable);
     end else derivate2 := 0;
     ErrorAt:=ErrorPosition;
end; {Derivate2}

function Solve(const What,Variable:tString;var guessed1,guessed2:tRealNumber;var ErrorAt:byte;var cycl:longint):tRealNumber;
var guess2,PrmValue:tRealNumber;
    b,WasParameter:boolean;
begin;
  WasParameter:=IsParameter(Variable);
  if WasParameter then PrmValue:=GetParameter(Variable,b);
  if derivate(What,Variable,guessed1,ErrorAt)=0.0 then
     begin;
           PrepareError(ErrCanNotBeSolved,1,What);
           solve:=0;
           exit;
     end;
  inc(cycl);
  if cycl>maxcycles then begin;PrepareError(ErrCanNotBeSolved,1,What);solve:=0;exit;end;
  IntoParameters(Variable,guessed1);
  guess2:=guessed1-(Evaluate(What,b)/derivate(What,Variable,guessed1,ErrorAt));
  if ErrorNumber<>ErrOK then begin;solve:=0;ErrorAt:=ErrorPosition;exit;end;
{  if (guess2>guessed2)and(guess2<guessed1) then begin;PrepareError(ErrCanNotBeSolved,1,What);solve:=0;exit;end;
  if (guess2<guessed2)and(guess2>guessed1) then begin;PrepareError(ErrCanNotBeSolved,1,What);solve:=0;exit;end;}
  if abs(guessed1-guess2)<=maxdelta then solve:=guess2
                                 else solve:=solve(What,Variable,guess2,guessed1,ErrorAt,cycl);
  if WasParameter then IntoParameters(Variable,PrmValue)
                  else DeleteParameter(Variable);
end;

function Solver(const What,Variable:tString;var ErrorAt:byte):tRealNumber;
var guess,Result,PrmValue:tRealNumber;
    Cycles:longint;
    b,WasParameter:boolean;
    TempErrNumber,TempErrPosition:byte; {cislo chyby, pozice chyby}
    TempErrString:string; {tString, ktery zpusobil chybu}
begin;
  WasParameter:=IsParameter(Variable);
  if WasParameter then PrmValue:=GetParameter(Variable,b);
WasSolve:=True;
SolveResult:=0.0;
randomize;
guess:=0;
Cycles:=0;
{DeleteAllParameters;}
IntoParameters(Variable,guess);
if Evaluate(What,b)=0 then begin;solver:=0;ErrorAt:=0;WasSolve:=True;exit;end;
if (derivate(What,Variable,guess,ErrorAt)=0) then
begin;
guess:=1;
{DeleteAllParameters;}
IntoParameters(Variable,guess);
if Evaluate(What,b)=0 then begin;solver:=1;ErrorAt:=0;WasSolve:=True;exit;end;
if (derivate(What,Variable,guess,ErrorAt)=0) then
begin;
guess:=-1;
{DeleteAllParameters;}
IntoParameters(Variable,guess);
if Evaluate(What,b)=0 then begin;solver:=-1;ErrorAt:=0;WasSolve:=True;exit;end;
end;
end;
while (derivate(What,Variable,guess,ErrorAt)=0)and(Cycles<maxcycles)
      and(ErrorNumber in CanBeInSolver) do
begin;
guess:=random(maxcycles);
if (derivate(What,Variable,guess,ErrorAt)=0) then guess:=-guess;
inc(Cycles);
end;
if not(ErrorNumber in CanBeInSolver) then PrepareError(ErrBadExpression,1,What)
else
if Cycles>=maxcycles then PrepareError(ErrCanNotBeSolved,1,What)
else
begin;
  Cycles:=0;
  Result:=solve(What,Variable,guess,guess,ErrorAt,Cycles);
  solver:=Result;
  IntoParameters(Variable,Result);
    TempErrNumber:= ErrorNumber;
    TempErrPosition:=ErrorPosition;
    TempErrString:= ErrorString;
  SolveResult:=Evaluate(What,b);
    ErrorNumber:= TempErrNumber;
    ErrorPosition:=TempErrPosition;
    ErrorString:= TempErrString;
end;
WasSolve:=True;
{DeleteAllParameters;}
  if WasParameter then IntoParameters(Variable,PrmValue)
                  else DeleteParameter(Variable);
end;

function Solverg(const What,Variable:tString;var guess:tRealNumber;var ErrorAt:byte):tRealNumber;
var Cycles:longint;
    Result,PrmValue:tRealNumber;
    b,WasParameter:boolean;
    TempErrNumber,TempErrPosition:byte; {cislo chyby, pozice chyby}
    TempErrString:string; {tString, ktery zpusobil chybu}
begin;
  WasParameter:=IsParameter(Variable);
  if WasParameter then PrmValue:=GetParameter(Variable,b);
WasSolve:=True;
SolveResult:=0.0;
Cycles:=0;
{DeleteAllParameters;}
IntoParameters(Variable,guess);
if Evaluate(What,b)=0 then begin;WasSolve:=True;solverg:=guess;ErrorAt:=0;exit;end;
Cycles:=0;
Result:=solve(What,Variable,guess,guess,ErrorAt,Cycles);
  solverg:=Result;
  IntoParameters(Variable,Result);
    TempErrNumber:= ErrorNumber;
    TempErrPosition:=ErrorPosition;
    TempErrString:= ErrorString;
  SolveResult:=Evaluate(What,b);
    ErrorNumber:= TempErrNumber;
    ErrorPosition:=TempErrPosition;
    ErrorString:= TempErrString;
{DeleteAllParameters;}
  if WasParameter then IntoParameters(Variable,PrmValue)
                  else DeleteParameter(Variable);
WasSolve:=True;
end;

{$F+  musi byt far, protoze pouzivam jako typ procedura
 ****************************************************************************
 *                                Zacatek Funkci                            *
 ****************************************************************************}
(****************************************************************************
 *                                EvalRandom                            *
 ****************************************************************************)
procedure EvalRandom(var Result:tRealNumber;var Express:tExpression);
begin
	Result := random;
end; {EvalRandom}

(****************************************************************************
 *                                EvalTrunc                               *
 ****************************************************************************)
procedure EvalTrunc(var Result : tRealNumber;var Express:tExpression);
begin
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     Result := realtrunc(Result);
end; {EvalTrunc}

(****************************************************************************
 *                                EvalCeil                               *
 ****************************************************************************)
procedure EvalCeil(var Result : tRealNumber;var Express:tExpression);
begin
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     Result := RealCeil(Result);
end; {EvalCeil}

(****************************************************************************
 *                                 EvalFrac                              *
 ****************************************************************************)
procedure EvalFrac(var Result : tRealNumber;var Express:tExpression);
begin
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     Result := frac(Result);
end; {EvalFrac}

(****************************************************************************
 *                            EvalRound                             *
 ****************************************************************************)
procedure EvalRound(var Result : tRealNumber;var Express:tExpression);
begin
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     Result := RealRound(Result);
end; {EvalRound}

(****************************************************************************
 *                             EvalPower                               *
 ****************************************************************************)
procedure EvalPower(var Result : tRealNumber;var Express:tExpression);
var
   Result1,Result2:tRealNumber;
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetTwoParameters(Result1,Result2,express);
     if ErrorNumber=ErrOK then
     if (abs(Result2-frac(Result2))=Result2)or
        (((Result2-frac(Result2))=Result2)and(Result1<>0))or
        ((Result2<0)and(Result1>0))or
        ((Result2>0)and(Result1>=0))
        then
        begin;
        if testp(Result1,Result2) then  Result := power(Result1,Result2)
                              else PrepareError(ErrOverflow,TempPosition,'');
        end
        else
        begin;
         if (Result1=0)and(Result2<0) then PrepareError(ErrDiv0,TempPosition,'') else
         if (Result1<0) then PrepareError(ErrBadPower,TempPosition,'');
        end;
end; {EvalPower}

(****************************************************************************
 *                              EvalPower2                             *
 ****************************************************************************)
procedure EvalPower2(var Result : tRealNumber;var Express:tExpression);
var
   TempResult:tRealNumber;
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(TempResult,express);
     if ErrorNumber=ErrOK then
     begin;
     if testp2(TempResult) then Result := power2(TempResult)
                     else PrepareError(ErrOverflow,TempPosition,'');
     end
end; {EvalPower2}

(****************************************************************************
 *                              EvalPower10                            *
 ****************************************************************************)
procedure EvalPower10(var Result : tRealNumber;var Express:tExpression);
var
   TempResult:tRealNumber;
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(TempResult,express);
     if ErrorNumber=ErrOK then
     if testp10(TempResult) then Result := power10(TempResult)
                      else PrepareError(ErrOverflow,TempPosition,'');
end; {EvalPower10}

(****************************************************************************
 *                                 EvalExp                               *
 ****************************************************************************)
procedure EvalExp(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     if testexp(Result) then Result := exp(Result)
                      else PrepareError(ErrOverflow,TempPosition,'');
end; {EvalExp}

(****************************************************************************
 *                                   EvalLn                              *
 ****************************************************************************)
procedure EvalLn(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     if (Result > 0.0) then Result := ln(Result)
        else PrepareError(ErrBadLog,TempPosition,Number2String(Result));
end; {EvalLn}

(****************************************************************************
 *                                  EvalLog10                            *
 ****************************************************************************)
procedure EvalLog10(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     if (Result > 0.0) then Result := log10(Result)
        else PrepareError(ErrBadLog,TempPosition,Number2String(Result));
end; {EvalLog10}

(****************************************************************************
 *                                 EvalLog                               *
 ****************************************************************************)
procedure EvalLog(var Result : tRealNumber;var Express:tExpression);
var
   Result1,Result2:tRealNumber;
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetTwoParameters(Result1,Result2,express);
     if ErrorNumber=ErrOK then
     begin;
     if (Result1 > 0.0)and(Result2 > 0.0)and(Result1 <> 1.0) then Result := ln(Result2)/ln(Result1)
        else
        begin;
        if (Result2 <= 0.0) then
           PrepareError(ErrBadLog,TempPosition,Number2String(Result2)) else
        if (Result1 <= 0.0)or(Result1 = 1.0) then
           PrepareError(ErrBadLogBase,TempPosition,Number2String(Result1));
        end;
     end;
end; {EvalLog}

(****************************************************************************
 *                              EvalMaxAbs                               *
 ****************************************************************************)
procedure EvalMaxAbs(var Result : tRealNumber;var Express:tExpression);
var
   Result1,Result2:tRealNumber;
begin
     GetTwoParameters(Result1,Result2,express);
     if ErrorNumber=ErrOK then
     begin;
     if abs(Result1)>abs(Result2) then Result:=Result1 else Result:=Result2;
     end;
end; {EvalMaxAbs}

(****************************************************************************
 *                                 EvalMax                               *
 ****************************************************************************)
procedure EvalMax(var Result : tRealNumber;var Express:tExpression);
var
   Result1,Result2:tRealNumber;
begin
     GetTwoParameters(Result1,Result2,express);
     if ErrorNumber=ErrOK then
     begin;
     if Result1>Result2 then Result:=Result1 else Result:=Result2;
     end;
end; {EvalMax}

(****************************************************************************
 *                              EvalAvg                               *
 ****************************************************************************)
procedure EvalAvg(var Result : tRealNumber;var Express:tExpression);
var
   Result1,Result2:tRealNumber;
begin
     GetTwoParameters(Result1,Result2,Express);
     if ErrorNumber=ErrOK then
     begin;
     Result:=(Result1+Result2)/2
     end;
end; {EvalAvg}

(****************************************************************************
 *                              EvalAvgG                              *
 ****************************************************************************)
procedure EvalAvgG(var Result : tRealNumber;var Express:tExpression);
var
   Result1,Result2:tRealNumber;
begin
     GetTwoParameters(Result1,Result2,Express);
     if ErrorNumber=ErrOK then
     begin;
     Result:=sqrt(abs(Result1*Result2))
     end;
end; {EvalAvgG}

(****************************************************************************
 *                              EvalAvgH                              *
 ****************************************************************************)
procedure EvalAvgH(var Result : tRealNumber;var Express:tExpression);
var
   Result1,Result2:tRealNumber;
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetTwoParameters(Result1,Result2,Express);
     if ErrorNumber=ErrOK then
     begin;
     if (Result1<>0)and(Result2<>0) then Result:=2/((1/Result1)+(1/Result2))
                                else PrepareError(ErrDiv0,TempPosition,'');
     end;
end; {EvalAvgH}

(****************************************************************************
 *                                 EvalMin                               *
 ****************************************************************************)
procedure EvalMin(var Result : tRealNumber;var Express:tExpression);
var
   Result1,Result2:tRealNumber;
begin
     GetTwoParameters(Result1,Result2,Express);
     if ErrorNumber=ErrOK then
     begin;
     if Result1<Result2 then Result:=Result1 else Result:=Result2;
     end;
end; {EvalMin}

(****************************************************************************
 *                              EvalMinAbs                               *
 ****************************************************************************)
procedure EvalMinAbs(var Result : tRealNumber;var Express:tExpression);
var
   Result1,Result2:tRealNumber;
begin
     GetTwoParameters(Result1,Result2,Express);
     if ErrorNumber=ErrOK then
     begin;
     if abs(Result1)<abs(Result2) then Result:=Result1 else Result:=Result2;
     end;
end; {EvalMinAbs}

(****************************************************************************
 *                              EvalLog2                                 *
 ****************************************************************************)
procedure EvalLog2(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     if (Result > 0.0) then Result := log2(Result)
        else PrepareError(ErrBadLog,TempPosition,Number2String(Result));
end; {EvalLog2}

(****************************************************************************
 *                             EvalInv                              *
 ****************************************************************************)
procedure EvalInv(var Result : tRealNumber;var Express:tExpression);
var TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
     if Result<>0 then Result := 1/Result
                    else PrepareError(ErrDiv0,TempPosition,'');
     end;
end; {EvalInv}

(****************************************************************************
 *                                EvalAbs                                *
 ****************************************************************************)
procedure EvalAbs(var Result : tRealNumber;var Express:tExpression);
begin
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     Result := Abs(Result);
end; {EvalAbs}

(****************************************************************************
 *                                EvalSign                                *
 ****************************************************************************)
procedure EvalSign(var Result : tRealNumber;var Express:tExpression);
begin
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     Result := Sign(Result);
end; {EvalSign}

(****************************************************************************
 *                                EvalDerivate                           *
 ****************************************************************************)
procedure EvalDerivate(var Result : tRealNumber;var Express:tExpression);
var b:boolean;
    Result1,Result2:tRealNumber;
    TempPosition,i,endp,startp,temperr:byte;
    s:tString;
begin
     IgnoreSpaces(express);
     if (Express.Expression [Express.Position] <> '(') then
        begin;
        PrepareError(ErrNoBracketStart,Express.Position,'');
        Result := 0;
        exit;
        end;
     inc(Express.Position);
     s:=StringUpCase(ReadSubExpression(express));
     GetCurrentParameter(Result,Express);
     if (Express.Expression [Express.Position] = ',') then inc(Express.Position);
     TempPosition:=Express.Position-1;
     if ErrorNumber=ErrOK then
     begin;
     endp:=0;
     startp:=1;
     i:=express.position;
     while endp<startp do
     begin;
       if express.expression[i]='(' then inc(startp) else
       if express.expression[i]=')' then inc(endp);
       inc(i);
     end;
        Result:=derivate(copy(Express.Expression,express.position,i-1-express.position),s,Result,temperr);
        if ErrorNumber<>ErrOK then ErrorPosition:=TempPosition+temperr;
     end else Result := 0;
     Express.Position:=i;
end; {EvalDerivate}

(****************************************************************************
 *                               EvalDerivate2                           *
 ****************************************************************************)
procedure EvalDerivate2(var Result : tRealNumber;var Express:tExpression);
var b:boolean;
    Result1,Result2:tRealNumber;
    TempPosition,i,endp,startp,temperr:byte;
    s:tString;
begin
     IgnoreSpaces(express);
     if (Express.Expression [Express.Position] <> '(') then
        begin;
        PrepareError(ErrNoBracketStart,Express.Position,'');
        Result := 0;
        exit;
        end;
     inc(Express.Position);
     s:=StringUpCase(ReadSubExpression(express));
     GetCurrentParameter(Result,Express);
     if (Express.Expression [Express.Position] = ',') then inc(Express.Position);
     TempPosition:=Express.Position-1;
     if ErrorNumber=ErrOK then
     begin;
     endp:=0;
     startp:=1;
     i:=express.position;
     while endp<startp do
     begin;
       if express.expression[i]='(' then inc(startp) else
       if express.expression[i]=')' then inc(endp);
       inc(i);
     end;
        Result:=derivate2(copy(Express.Expression,express.position,i-1-express.position),s,Result,temperr);
        if ErrorNumber<>ErrOK then ErrorPosition:=TempPosition+temperr;
     end else Result := 0;
     Express.Position:=i;
end; {EvalDerivate2}

(****************************************************************************
 *                                EvalSum                               *
 ****************************************************************************)
procedure EvalSum(var Result : tRealNumber;var Express:tExpression);
var b:boolean;
    Result1,Result2:tRealNumber;
    TempPosition,i,endp,startp,temperr:byte;
    s:tString;
begin
     IgnoreSpaces(express);
     if (Express.Expression [Express.Position] <> '(') then
        begin;
        PrepareError(ErrNoBracketStart,Express.Position,'');
        Result := 0;
        exit;
        end;
     inc(Express.Position);
     s:=StringUpCase(ReadSubExpression(express));
     GetCurrentParameter(Result1,Express);
     if (Express.Expression [Express.Position] = ',') then inc(Express.Position);
     GetCurrentParameter(Result2,Express);
     if (Express.Expression [Express.Position] = ',') then inc(Express.Position);
     TempPosition:=Express.Position-1;
     if ErrorNumber=ErrOK then
     begin;
     endp:=0;
     startp:=1;
     i:=express.position;
     while endp<startp do
     begin;
       if express.expression[i]='(' then inc(startp) else
       if express.expression[i]=')' then inc(endp);
       inc(i);
     end;
        Result:=Sum(copy(Express.Expression,express.position,i-1-express.position),s,Result1,Result2,temperr);
        if ErrorNumber<>ErrOK then ErrorPosition:=TempPosition+temperr;
     end else Result := 0;
     Express.Position:=i;
end; {EvalSum}

(****************************************************************************
 *                                EvalSumTable                               *
 ****************************************************************************)
procedure EvalSumTable(var Result : tRealNumber;var Express:tExpression);
var b:boolean;
    Result1,Result2:tRealNumber;
    TempPosition,i,endp,startp,temperr:byte;
    s:tString;
begin
     IgnoreSpaces(express);
     if (Express.Expression [Express.Position] <> '(') then
        begin;
        PrepareError(ErrNoBracketStart,Express.Position,'');
        Result := 0;
        exit;
        end;
     inc(Express.Position);
     s:=StringUpCase(ReadSubExpression(express));
     GetCurrentParameter(Result1,Express);
     if (Express.Expression [Express.Position] = ',') then inc(Express.Position);
     GetCurrentParameter(Result2,Express);
     if (Express.Expression [Express.Position] = ',') then inc(Express.Position);
     TempPosition:=Express.Position-1;
     if ErrorNumber=ErrOK then
     begin;
     endp:=0;
     startp:=1;
     i:=express.position;
     while endp<startp do
     begin;
       if express.expression[i]='(' then inc(startp) else
       if express.expression[i]=')' then inc(endp);
       inc(i);
     end;
        Result:=SumTable(copy(Express.Expression,express.position,i-1-express.position),s,Result1,Result2,temperr);
        if ErrorNumber<>ErrOK then ErrorPosition:=TempPosition+temperr;
     end else Result := 0;
     Express.Position:=i;
end; {EvalSumTable}

(****************************************************************************
 *                                EvalIntg                               *
 ****************************************************************************)
procedure EvalIntg(var Result : tRealNumber;var Express:tExpression);
var b:boolean;
    Result1,Result2:tRealNumber;
    TempPosition,i,endp,startp,temperr:byte;
    s:tString;
begin
     IgnoreSpaces(express);
     if (Express.Expression [Express.Position] <> '(') then
        begin;
        PrepareError(ErrNoBracketStart,Express.Position,'');
        Result := 0;
        exit;
        end;
     inc(Express.Position);
     s:=StringUpCase(ReadSubExpression(express));
     GetCurrentParameter(Result1,Express);
     if (Express.Expression [Express.Position] = ',') then inc(Express.Position);
     GetCurrentParameter(Result2,Express);
     if (Express.Expression [Express.Position] = ',') then inc(Express.Position);
     TempPosition:=Express.Position-1;
     if ErrorNumber=ErrOK then
     begin;
     endp:=0;
     startp:=1;
     i:=express.position;
     while endp<startp do
     begin;
       if express.expression[i]='(' then inc(startp) else
       if express.expression[i]=')' then inc(endp);
       inc(i);
     end;
        Result:=Intg(copy(Express.Expression,express.position,i-1-express.position),s,Result1,Result2,temperr);
        if ErrorNumber<>ErrOK then ErrorPosition:=TempPosition+temperr;
     end else Result := 0;
     Express.Position:=i;
end; {EvalIntg}

(****************************************************************************
 *                                EvalIntgS                              *
 ****************************************************************************)
procedure EvalIntgS(var Result : tRealNumber;var Express:tExpression);
var b:boolean;
    Result1,Result2:tRealNumber;
    TempPosition,i,endp,startp,temperr:byte;
    s:tString;
begin
     IgnoreSpaces(express);
     if (Express.Expression [Express.Position] <> '(') then
        begin;
        PrepareError(ErrNoBracketStart,Express.Position,'');
        Result := 0;
        exit;
        end;
     inc(Express.Position);
     s:=StringUpCase(ReadSubExpression(express));
     GetCurrentParameter(Result1,Express);
     if (Express.Expression [Express.Position] = ',') then inc(Express.Position);
     GetCurrentParameter(Result2,Express);
     if (Express.Expression [Express.Position] = ',') then inc(Express.Position);
     TempPosition:=Express.Position-1;
     if ErrorNumber=ErrOK then
     begin;
     endp:=0;
     startp:=1;
     i:=express.position;
     while endp<startp do
     begin;
       if express.expression[i]='(' then inc(startp) else
       if express.expression[i]=')' then inc(endp);
       inc(i);
     end;
        Result:=IntgS(copy(Express.Expression,express.position,i-1-express.position),s,Result1,Result2,temperr);
        if ErrorNumber<>ErrOK then ErrorPosition:=TempPosition+temperr;
     end else Result := 0;
     Express.Position:=i;
end; {EvalIntgS}

(****************************************************************************
 *                                EvalSolveGuess                             *
 ****************************************************************************)
procedure EvalSolveGuess(var Result : tRealNumber;var Express:tExpression);
var b:boolean;
    Result1,Result2,guess:tRealNumber;
    TempPosition,i,endp,startp,temperr:byte;
    s:string;
begin
     IgnoreSpaces(express);
     if (Express.Expression [Express.Position] <> '(') then
        begin;
        PrepareError(ErrNoBracketStart,Express.Position,'');
        Result := 0;
        exit;
        end;
     inc(Express.Position);
     s:=StringUpCase(ReadSubExpression(express));
     GetCurrentParameter(Result,Express);
     if (Express.Expression [Express.Position] = ',') then inc(Express.Position);
     TempPosition:=Express.Position-1;
     if ErrorNumber=ErrOK then
     begin;
     endp:=0;
     startp:=1;
     i:=express.position;
     while endp<startp do
     begin;
       if express.expression[i]='(' then inc(startp) else
       if express.expression[i]=')' then inc(endp);
       inc(i);
     end;
        Result:=solverg(copy(Express.Expression,express.position,i-1-express.position),s,Result,temperr);
        if ErrorNumber<>ErrOK then ErrorPosition:=TempPosition+temperr;
     end else Result := 0;
     Express.Position:=i;
end; {EvalSolveGuess}

(****************************************************************************
 *                                EvalSolve                              *
 ****************************************************************************)
procedure EvalSolve(var Result : tRealNumber;var Express:tExpression);
var b:boolean;
    Result1,Result2,guess:tRealNumber;
    TempPosition,i,endp,startp,temperr:byte;
    s:string;
begin
     IgnoreSpaces(express);
     if (Express.Expression [Express.Position] <> '(') then
        begin;
        PrepareError(ErrNoBracketStart,Express.Position,'');
        Result := 0;
        exit;
        end;
     inc(Express.Position);
     s:=StringUpCase(ReadSubExpression(express));
     if (Express.Expression [Express.Position] = ',') then inc(Express.Position);
     TempPosition:=Express.Position-1;
     if ErrorNumber=ErrOK then
     begin;
     endp:=0;
     startp:=1;
     i:=express.position;
     while endp<startp do
     begin;
       if express.expression[i]='(' then inc(startp) else
       if express.expression[i]=')' then inc(endp);
       inc(i);
     end;
        Result:=solver(copy(Express.Expression,express.position,i-1-express.position),s,temperr);
        if ErrorNumber<>ErrOK then ErrorPosition:=TempPosition+temperr;
     end else Result := 0;
     Express.Position:=i;
end; {EvalSolve}

(****************************************************************************
 *                               EvalSin                                 *
 ****************************************************************************)
procedure EvalSin(var Result : tRealNumber;var Express:tExpression);
begin
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
     while ToRad(Result)>(pi_10000) do Result:=Result-FromRad(pi_10000);
     while ToRad(Result)>(pi_1000) do Result:=Result-FromRad(pi_1000);
     while ToRad(Result)>(pi_100) do Result:=Result-FromRad(pi_100);
     while ToRad(Result)>(pi_10) do Result:=Result-FromRad(pi_10);
     while ToRad(Result)>(pi_2) do Result:=Result-FromRad(pi_2);
     while ToRad(Result)<(-pi_10000) do Result:=Result+FromRad(pi_10000);
     while ToRad(Result)<(-pi_1000) do Result:=Result+FromRad(pi_1000);
     while ToRad(Result)<(-pi_100) do Result:=Result+FromRad(pi_100);
     while ToRad(Result)<(-pi_10) do Result:=Result+FromRad(pi_10);
     while ToRad(Result)<(-pi_2) do Result:=Result+FromRad(pi_2);
     Result := sin(ToRad(Result));
     end;
end; {EvalSin}

(****************************************************************************
 *                                EvalCos                                *
 ****************************************************************************)
procedure EvalCos(var Result : tRealNumber;var Express:tExpression);
begin
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
     while ToRad(Result)>(pi_10000) do Result:=Result-FromRad(pi_10000);
     while ToRad(Result)>(pi_1000) do Result:=Result-FromRad(pi_1000);
     while ToRad(Result)>(pi_100) do Result:=Result-FromRad(pi_100);
     while ToRad(Result)>(pi_10) do Result:=Result-FromRad(pi_10);
     while ToRad(Result)>(pi_2) do Result:=Result-FromRad(pi_2);
     while ToRad(Result)<(-pi_10000) do Result:=Result+FromRad(pi_10000);
     while ToRad(Result)<(-pi_1000) do Result:=Result+FromRad(pi_1000);
     while ToRad(Result)<(-pi_100) do Result:=Result+FromRad(pi_100);
     while ToRad(Result)<(-pi_10) do Result:=Result+FromRad(pi_10);
     while ToRad(Result)<(-pi_2) do Result:=Result+FromRad(pi_2);
     Result := sin(ToRad(Result)+(pi_pul));
     end;
end; {EvalCos}

(****************************************************************************
 *                                EvalTan                                *
 ****************************************************************************)
procedure EvalTan(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
     while ToRad(Result)>(pi_10000) do Result:=Result-FromRad(pi_10000);
     while ToRad(Result)>(pi_1000) do Result:=Result-FromRad(pi_1000);
     while ToRad(Result)>(pi_100) do Result:=Result-FromRad(pi_100);
     while ToRad(Result)>(pi_10) do Result:=Result-FromRad(pi_10);
     while ToRad(Result)>(pi_2) do Result:=Result-FromRad(pi_2);
     while ToRad(Result)<(-pi_10000) do Result:=Result+FromRad(pi_10000);
     while ToRad(Result)<(-pi_1000) do Result:=Result+FromRad(pi_1000);
     while ToRad(Result)<(-pi_100) do Result:=Result+FromRad(pi_100);
     while ToRad(Result)<(-pi_10) do Result:=Result+FromRad(pi_10);
     while ToRad(Result)<(-pi_2) do Result:=Result+FromRad(pi_2);
     while ToRad(Result)>(pi) do Result:=Result-FromRad(pi);
     while ToRad(Result)<(-pi) do Result:=Result+FromRad(pi);
     if ( cos(ToRad(Result)) <> 0 ) then
	Result := tan(ToRad(Result))
     else PrepareError(ErrBadTan,TempPosition,Number2String(Result));
     end;
end; {EvalTan}

(****************************************************************************
 *                                EvalCoTan                              *
 ****************************************************************************)
procedure EvalCoTan(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
     while ToRad(Result)>(pi_10000) do Result:=Result-FromRad(pi_10000);
     while ToRad(Result)>(pi_1000) do Result:=Result-FromRad(pi_1000);
     while ToRad(Result)>(pi_100) do Result:=Result-FromRad(pi_100);
     while ToRad(Result)>(pi_10) do Result:=Result-FromRad(pi_10);
     while ToRad(Result)>(pi_2) do Result:=Result-FromRad(pi_2);
     while ToRad(Result)<(-pi_10000) do Result:=Result+FromRad(pi_10000);
     while ToRad(Result)<(-pi_1000) do Result:=Result+FromRad(pi_1000);
     while ToRad(Result)<(-pi_100) do Result:=Result+FromRad(pi_100);
     while ToRad(Result)<(-pi_10) do Result:=Result+FromRad(pi_10);
     while ToRad(Result)<(-pi_2) do Result:=Result+FromRad(pi_2);
     while ToRad(Result)>(pi) do Result:=Result-FromRad(pi);
     while ToRad(Result)<(-pi) do Result:=Result+FromRad(pi);
     if ( sin(ToRad(Result)) <> 0 ) then
	Result := cos(ToRad(Result)) / sin(ToRad(Result))
     else PrepareError(ErrBadCoTan,TempPosition,Number2String(Result));
     end;
end; {EvalCoTan}

(****************************************************************************
 *                             EvalArcSin                                *
 ****************************************************************************)
procedure EvalArcSin(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
     if (abs(Result) < 1.0) then
	Result := FromRad(arcTan(Result/sqrt(1-Result*Result)))
        else
        if Result=1.0 then
        Result:=FromRad(pi_pul)
        else
        if Result=-1.0 then
        Result:=FromRad(-pi_pul)
     else PrepareError(ErrBadArcSin,TempPosition,Number2String(Result));
     end;
end; {EvalArcSin}

(****************************************************************************
 *                               EvalArcCos                              *
 ****************************************************************************)
procedure EvalArcCos(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
	if ((Result <> 0.0) and (abs(Result) < 1.0)) then
	 Result := FromRad(arcTan(sqrt(1-Result*Result)/Result))
         else if Result = 0.0 then Result:=FromRad(pi_pul)
         else if Result=1.0 then Result:=FromRad(0.0)
         else if Result=(-1.0) then Result:=FromRad(-(pi))
         else PrepareError(ErrBadArcCos,TempPosition,Number2String(Result));
     end;
end; {EvalArcCos}

(****************************************************************************
 *                               EvalArcTan                              *
 ****************************************************************************)
procedure EvalArcTan(var Result : tRealNumber;var Express:tExpression);
begin
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     Result := FromRad(ArcTan(Result));
end; {EvalArcTan}

(****************************************************************************
 *                              EvalArcCoTan                             *
 ****************************************************************************)
procedure EvalArcCoTan(var Result : tRealNumber;var Express:tExpression);
begin
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     Result := FromRad((pi_pul)-ArcTan(Result));
end; {EvalArcTan}

(****************************************************************************
 *                               EvalSinH                                *
 ****************************************************************************)
procedure EvalSinH(var Result : tRealNumber;var Express:tExpression);
var TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
     if abs(Result)<=11356 then Result := (1/2)*(exp(Result)-exp(-Result))
                            else PrepareError(ErrOverflow,TempPosition,'');
     end;
end; {EvalSin}

(****************************************************************************
 *                                EvalCosH                               *
 ****************************************************************************)
procedure EvalCosH(var Result : tRealNumber;var Express:tExpression);
var TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
     if abs(Result)<=11356 then Result := (1/2)*(exp(Result)+exp(-Result))
                            else PrepareError(ErrOverflow,TempPosition,'');
     end;
end; {EvalCos}

(****************************************************************************
 *                                EvalTanH                               *
 ****************************************************************************)
procedure EvalTanH(var Result : tRealNumber;var Express:tExpression);
begin
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     	Result := (exp(Result)-exp(-Result))/(exp(Result)+exp(-Result));
end; {EvalTan}

(****************************************************************************
 *                                EvalCoTanH                             *
 ****************************************************************************)
procedure EvalCoTanH(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     if ( Result <> 0 )and((exp(Result)-exp(-Result))<>0) then
	Result := (exp(Result)+exp(-Result))/(exp(Result)-exp(-Result))
        else PrepareError(ErrBadCoTanH,TempPosition,Number2String(Result));
end; {EvalCoTan}

(****************************************************************************
 *                             EvalArgSinH                               *
 ****************************************************************************)
procedure EvalArgSinH(var Result : tRealNumber;var Express:tExpression);
begin
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     Result:=ln(Result+sqrt((Result*Result)+1));
end; {EvalArcSin}

(****************************************************************************
 *                               EvalArgCosH                             *
 ****************************************************************************)
procedure EvalArgCosH(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
	if (Result >= 1.0) then
           Result:=ln(Result+sqrt((Result*Result)-1))
         else PrepareError(ErrBadArgCosH,TempPosition,Number2String(Result));
     end;
end; {EvalArcCos}

(****************************************************************************
 *                               EvalArgTanH                             *
 ****************************************************************************)
procedure EvalArgTanH(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
	if (abs(Result) < 1.0) then
           Result:=ln(sqrt( (Result+1)/(1-Result) ))
         else PrepareError(ErrBadArgTanH,TempPosition,Number2String(Result));
     end;
end; {EvalArcTan}

(****************************************************************************
 *                              EvalArgCoTanH                            *
 ****************************************************************************)
procedure EvalArgCoTanH(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
	if (abs(Result) > 1.0) then
           Result:=ln(sqrt( (Result+1)/(Result-1) ))
         else PrepareError(ErrBadArgCoTanH,TempPosition,Number2String(Result));
     end;
end; {EvalArcTan}

(****************************************************************************
 *                              EvalSqr                             *
 ****************************************************************************)
procedure EvalSqr(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
     if testsqr(Result) then Result := Sqr(Result)
                           else PrepareError(ErrOverflow,TempPosition,'');
     end;
end; {EvalSqr}

(****************************************************************************
 *                             EvalSqrt                            *
 ****************************************************************************)
procedure EvalSqrt(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
     if Result>=0 then Result := Sqrt(Result)
                    else PrepareError(ErrBadRoot,TempPosition,Number2String(Result));
     end;
end; {EvalSqrt}

(****************************************************************************
 *                              EvalCub                             *
 ****************************************************************************)
procedure EvalCub(var Result : tRealNumber;var Express:tExpression);
var
   TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     if testnas(Result,Result) then
     begin;
     if testnas(Result*Result,Result) then Result := Result*Result*Result
                                            else PrepareError(ErrOverflow,TempPosition,'');
     end
                           else PrepareError(ErrOverflow,TempPosition,'');
end; {EvalCub}

(****************************************************************************
 *                             EvalCubR                            *
 ****************************************************************************)
procedure EvalCubR(var Result : tRealNumber;var Express:tExpression);
begin
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     begin;
     if Result<0 then Result := -Power(abs(Result),1/3)
                   else Result := Power(Result,1/3);
     end;
end; {EvalCubR}

(****************************************************************************
 *                             EvalComb                             *
 ****************************************************************************)
procedure EvalComb(var Result : tRealNumber;var Express:tExpression);
var Result1,Result2:tRealNumber;
    TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetTwoParameters(Result1,Result2,Express);
     if ErrorNumber=ErrOK then
     if Result1<=Result2 then
     begin;
     if testfac(Result2) then Result := fact(Result2)/(fact(Result1)*fact(Result2-Result1))
                       else PrepareError(ErrOverflow,TempPosition,'');
     end
                     else PrepareError(ErrBadComb,TempPosition,Number2String(Result1));
end; {EvalComb}

(****************************************************************************
 *                             EvalVar                               *
 ****************************************************************************)
procedure EvalVar(var Result : tRealNumber;var Express:tExpression);
var Result1,Result2:tRealNumber;
    TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetTwoParameters(Result1,Result2,Express);
     if ErrorNumber=ErrOK then
     if Result1<=Result2 then
     begin;
     if testfac(Result2) then Result := fact(Result2)/(fact(Result2-Result1))
                       else PrepareError(ErrOverflow,TempPosition,'');
     end
                     else PrepareError(ErrBadVar,TempPosition,Number2String(Result1));
end; {EvalVar}

(****************************************************************************
 *                             EvalRad                                   *
 ****************************************************************************)
procedure EvalRad(var Result : tRealNumber;var Express:tExpression);
var TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     case AngleMeas of deg:if testnas(Result,180/pi) then Result:=(Result/pi)*180
                                                         else PrepareError(ErrOverflow,TempPosition,'');
                        rad:Result:=Result;
                        grad:if testnas(Result,200/pi) then Result:=(Result/pi)*200
                                                           else PrepareError(ErrOverflow,TempPosition,'');
                        end;
end; {EvalRad}

(****************************************************************************
 *                             EvalGrad                                  *
 ****************************************************************************)
procedure EvalGrad(var Result : tRealNumber;var Express:tExpression);
var TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     case AngleMeas of deg:if testnas(Result,9/10) then Result:=(Result/10)*9
                                                       else PrepareError(ErrOverflow,TempPosition,'');
                        rad:if testnas(Result,pi/200) then Result:=(Result/200)*pi
                                                        else PrepareError(ErrOverflow,TempPosition,'');
                        grad:Result:=Result;
                        end;
end; {EvalGrad}

(****************************************************************************
 *                             EvalDeg                                   *
 ****************************************************************************)
procedure EvalDeg(var Result : tRealNumber;var Express:tExpression);
var TempPosition:byte;
begin
     TempPosition:=Express.Position;
     GetSingleParameter(Result,Express);
     if ErrorNumber=ErrOK then
     case AngleMeas of deg:Result:=Result;
                        rad:if testnas(Result,180/pi) then Result:=(Result/180)*pi
                                                        else PrepareError(ErrOverflow,TempPosition,'');
                        grad:if testnas(Result,9/10) then Result:=(Result/9)*10
                                                       else PrepareError(ErrOverflow,TempPosition,'');
                        end;
end; {EvalDeg}
{$F-  musi byt far, protoze pouzivam jako typ procedura
 ****************************************************************************
 *                                Konec Funkci                              *
 ****************************************************************************}

{****************************************************************************
 *                             EvalFunction                                *
 ****************************************************************************
 Zjisti hodnotu aktualniho vyrazu - funkce,promenne,konstanty}
procedure EvalFunction(var Result : tRealNumber;var Express:tExpression);
var CurrentFunction:byte;
    Pocitano,b:boolean;
    TempPosition:byte;
    zjist:string;
begin
Pocitano:=false;
     TempPosition:=Express.Position;
     zjist:=StringUpCase(ReadSubExpression(Express));
for CurrentFunction:=1 to MaxFunctions do
{   if StringUpCase(StringNoDivisor(copy(Express.Expression,Express.Position,15)))=FunctionData[CurrentFunction].Name then}
   if zjist=FunctionData[CurrentFunction].Name then
      begin;
{            Express.Position:=Express.Position+length(FunctionData[CurrentFunction].Name);}
            FunctionData[CurrentFunction].Calculate(Result,express);
            pocitano:=true;
            break;
      end;
if not pocitano then
   begin;
{     TempPosition:=Express.Position;
     zjist:=ReadSubExpression(Express);}
     if IsParameter(zjist) then Result:=GetParameter(zjist,b) else
     if IsVariable(zjist) then Result:=GetVariable(zjist,b) else
     if IsConstant(zjist) then Result:=GetConstant(zjist,b) else
                        PrepareError(ErrBadName,TempPosition,zjist);
   end;
end;

{****************************************************************************
 *                             Zakladni                                     *
 ****************************************************************************
 Zjisti jestli je prvni znak cislo nebo pismeno a podle toho spusti funkci}
procedure Zakladni(var Result : tRealNumber;var Express:tExpression);
var Result2:tRealNumber;
    TempPosition:byte;
begin
     if ErrorNumber=ErrOK then begin;
     if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'');
     case upcase(Express.Expression [Express.Position]) of '0'..'9','.','~'{,'-','+'}:Result := ReadNumber(Express);
                                    '#','$':Result := ReadNumBase(express);
                               'A'..'Z','_':EvalFunction(Result,express);
                               else
                               begin;
                               if Express.Expression[Express.Position] in Divisors then
                               PrepareError(ErrNoValue,Express.Position,ReadSubExpression(express))
                               else
                               PrepareError(ErrBadName,Express.Position,ReadSubExpression(express));
                               end;
                               end;
     IgnoreSpaces(Express);
     end;
     TempPosition:=Express.Position;
     if (ErrorNumber=ErrOK)and
        (length(Express.Expression)>=Express.Position)and
        (UpCase(Express.Expression[Express.Position]) in ['0'..'9','.','~','#','$','A'..'Z','_'])
         then
        begin;
          Zakladni(Result2,Express);
          if (ErrorNumber=ErrOK) then
             begin;
               if testnas(Result,result2) then Result:=Result*Result2
                                          else PrepareError(ErrOverflow,TempPosition,'');
             end;
        end;
end;

{****************************************************************************
 *                             Priority7                                    *
 ****************************************************************************
 Zavorky}
procedure Priority7(var Result : tRealNumber;var Express:tExpression);
begin
     if (Express.Position<=length(Express.Expression)) then
     if (Express.Expression [Express.Position] = '(') then begin
        Inc(Express.Position);
        IgnoreSpaces(Express);
        if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'');
        Priority2(Result,Express);
        IgnoreSpaces(Express); {dojki k )}
        if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'');
        if Express.Expression[Express.Position] <> ')' then PrepareError(ErrNoBracketEnd,Express.Position,'');
        Inc(Express.Position);
        IgnoreSpaces(Express);
     end else Zakladni (Result,Express);
end;

{****************************************************************************
 *                                 Priority6                                *
 ****************************************************************************
 Faktorial (!) do 1755}
procedure Priority6(var Result : tRealNumber;var Express:tExpression);
var TempPosition : byte;
begin
     TempPosition:=Express.Position;
     IgnoreSpaces(Express);
     if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'');
     Priority7(Result,Express);
     if ErrorNumber=ErrOK then begin
     IgnoreSpaces(Express);
     if (Express.Position<=length(Express.Expression)) then
     if (Result<0)and(Express.Expression [Express.Position]= '!')
        then PrepareError(ErrBadFactorial,TempPosition,Number2String(Result))
     else
     while (Express.Expression [Express.Position]= '!')
        and(Express.Position<=length(Express.Expression))
        and(ErrorNumber=ErrOK) do
      if testfac(Result) then
        begin;
          Result := fact(Result);
          Inc(Express.Position);
          IgnoreSpaces(Express);
        end
        else PrepareError(ErrOverflow,TempPosition,'');
     end;
end;

{****************************************************************************
 *                                 Priority5                                *
 ****************************************************************************
 Znamenko vyrazu (unarni +,-)}
procedure Priority5(var Result : tRealNumber;var Express:tExpression);
var op : Char;
    Code:byte;
    sign:trealNumber;
begin
  {new}
  sign:=1;
  if Express.Expression[Express.Position] in ['+','-'] then
  begin;
  if Express.Expression[Express.Position] ='-' then sign:=-1;
  Code:=1;
  while Express.Expression[Express.Position+Code] in [' ','+','-'] do
        begin;
        if Express.Expression[Express.Position+Code]='-' then sign:=sign*(-1);
        inc(Code);
        end;
  Express.Position:=Express.Position+Code;
  end;
  {endnew}

{     op := Express.Expression [Express.Position];
     if op in ['-','+'] then Inc(Express.Position);}
     IgnoreSpaces(Express);
     if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'');
     Priority6(Result,Express);
     if ErrorNumber=ErrOK then Result:=Result*sign;
{     if op = '-' then Result := - Result;}
end;

{****************************************************************************
 *                                 Priority4                                *
 ****************************************************************************
 mocnina}
procedure Priority4(var Result : tRealNumber;var Express:tExpression);
var Value2 : tRealNumber;
    TempPosition:byte;
begin
     TempPosition:=Express.Position;
     IgnoreSpaces(Express);
     if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'');
     Priority5(Result,Express);
     if ErrorNumber=ErrOK then
        if (Express.Position<=length(Express.Expression)) then
        if (Express.Expression [Express.Position] = '^')or
           ((Express.Expression [Express.Position]+Express.Expression [Express.Position+1])='**')
           then
        begin
           if Express.Expression [Express.Position]='*' then Inc(Express.Position);
           Inc(Express.Position);
           IgnoreSpaces(Express);
           if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'') else
           begin;
           Priority4(Value2,Express);
           if (abs(Value2-frac(Value2))=Value2)or
              (((Value2-frac(Value2))=Value2)and(Result<>0))or
              ((Value2<0)and(Result>0))or
              ((Value2>0)and(Result>=0))
              then
              begin;
                if testp(Result,Value2) then  Result := power(Result,Value2)
                                         else PrepareError(ErrOverflow,TempPosition,'');
              end
           else
              begin;
               if (Result=0)and(Value2<0) then PrepareError(ErrDiv0,TempPosition,'') else
               if (Result<0) then PrepareError(ErrBadPower,TempPosition,'');
              end;
           end;
        end;
end;

{****************************************************************************
 *                             Priority3                                    *
 ****************************************************************************
 nasobeni, deleni, celociselne deleni(%), zbytek po deleni(\)}
procedure Priority3(var Result : tRealNumber;var Express:tExpression);
var Value2 : tRealNumber;
    op   : Char;
    TempPosition:byte;
begin
     IgnoreSpaces(Express);
     if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'');
     Priority4(Result,Express);
     if ErrorNumber=ErrOK then begin
        IgnoreSpaces(Express);
        While (Express.Position<=length(Express.Expression))and
              ((Express.Expression [Express.Position] in ['*','/','%','\','(']) and
              not( (Express.Expression [Express.Position]+Express.Expression [Express.Position+1])='**')
               or(upcase(Express.Expression [Express.Position])+
                  upcase(Express.Expression [Express.Position+1])+
                  upcase(Express.Expression [Express.Position+2])='MOD')
               or(upcase(Express.Expression [Express.Position])+
                  upcase(Express.Expression [Express.Position+1])+
                  upcase(Express.Expression [Express.Position+2])='DIV')
               )
              and(Express.Position<=length(Express.Expression))and(ErrorNumber=ErrOK) do
        begin
         op := Express.Expression[Express.Position];
         if op<>'(' then Inc(Express.Position);
         if upcase(op) in ['M','D'] then Express.Position:=Express.Position+2;
         if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'') else
         begin;
         if op='(' then
         begin;
               IgnoreSpaces(Express);
               if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'');
               Priority6(Value2,Express);
               if ErrorNumber=ErrOK then
               Result:=Result*Value2;
         end else
         begin;
           TempPosition:=Express.Position;
           IgnoreSpaces(Express);
           if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'');
           Priority4(Value2,Express);
           if ErrorNumber=ErrOK then begin
           if op = '*' then
           begin;
           if testnas(Result,Value2) then Result := Result * Value2
           else PrepareError(ErrOverflow,TempPosition,'');
           end
              else begin
	        if (Value2 = 0.0) then PrepareError(ErrDiv0,TempPosition,'')
	      	else
                begin;
                if testnas(Result,1/Value2) then
                    case upcase(op) of '/':Result := Result / Value2;
                                       '%':Result := RealDiv(Result,Value2);
                                       '\':Result := RealMod(Result,Value2);
                                       'D':Result := RealDiv(Result,Value2);
                                       'M':Result := RealMod(Result,Value2);
                                      end
                else PrepareError(ErrOverflow,TempPosition,'');
                end;
	      end;
           IgnoreSpaces(Express);
           end;
           end; {if Express.Position>...}
         end;
        end; {while}
     end; {neni chyba}
end;

{****************************************************************************
 *                             Priority2                                    *
 ****************************************************************************
 scitani, odcitani}
procedure Priority2(var Result : tRealNumber;var Express:tExpression);
var Value2 : tRealNumber;
    op   : Char;
    TempPosition:byte;
begin
     IgnoreSpaces(Express);
     if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'');
     Priority3(Result,Express);
     if ErrorNumber=ErrOK then begin
        IgnoreSpaces(Express);
        While (Express.Position<=length(Express.Expression))
           and(Express.Expression [Express.Position] in ['+','-'])
           and(ErrorNumber=ErrOK) do begin
         op := Express.Expression [Express.Position];
         inc(Express.Position);
         if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'')
         else
         begin;
           IgnoreSpaces(Express);
           if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'');
           TempPosition:=Express.Position;
           Priority3(Value2,Express);
           if ErrorNumber=ErrOK then
           begin;
           if op = '+' then
           begin;
            if testscit(Result,Value2) then Result := Result + Value2
            else PrepareError(ErrOverflow,TempPosition,'');
           end else
           begin;
            if testscit(Result,-Value2) then Result := Result - Value2
            else PrepareError(ErrOverflow,TempPosition,'');
           end;
           IgnoreSpaces(Express);
           end; {chyba}
         end;
        end; {while}
     end; {neni chyba}
end;

{****************************************************************************
 *                             Priority1                                    *
 ****************************************************************************
 prirazeni}
procedure Priority1(var Result : tRealNumber;var Express:tExpression);
var temp : string;
    i:byte;
begin
     if pos('=',Express.Expression)>0 then
     begin;
       temp:=ReadSubExpression(Express);
       if pos('(',temp)>0 then
       begin;

       end else
       begin;
         for i:=1 to length(temp) do
             if temp[i] in (Divisors+['~']) then begin;
                                                PrepareError(ErrBadName,1,temp);
                                                break;
                                              end;
         Express.Position:=pos('=',Express.Expression)+1;
         IgnoreSpaces(Express);
         if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'');
         if ErrorNumber=ErrOK then Priority2(Result,Express);
         if ErrorNumber=ErrOK then IntoMemory(temp,Result);
         if MemoryError_<>0 then PrepareError(ErrMemory,1,temp);
       end;
     end {je =}
     else
     begin;
       IgnoreSpaces(Express);
       if (Express.Position>length(Express.Expression)) then PrepareError(ErrUnexpectedEnd,Express.Position,'');
       Priority2(Result,Express);
       if ErrorNumber=ErrOK then
       if (Express.Position<=length(Express.Expression)) then begin;
                                        IgnoreSpaces(Express);
                                        if (Express.Position<=length(Express.Expression))
                                           and(not(Express.Expression[Express.Position] in [' ',';'])) then
                                           PrepareError(ErrNoOperation,Express.Position,'');

                                      end;
     end;
end;

{****************************************************************************
 *                               Evaluate                                   *
 ****************************************************************************}
function Evaluate(What : tString;var IsValid:Boolean) : tRealNumber;
var Result : tRealNumber;
    Express:tExpression;
begin
     Express.Position := 1;
     Express.Expression:=What;
     WasSolve:=false;
     SolveResult:=0.0;
     ErrorNumber:=0;
     ErrorPosition:=0;
     ErrorString:='';
     {opravime pocet zavorek:}
     if CountChars('(',Express.Expression)>CountChars(')',Express.Expression) then
        Express.Expression:=
                Express.Expression+ReturnNChars(')',CountChars('(',Express.Expression)-CountChars(')',Express.Expression))
        else
         if CountChars('(',Express.Expression)<CountChars(')',Express.Expression) then PrepareError(ErrNoBracketStart,1,'');
     IgnoreSpaces(Express);
     Result := 0; {kdyz nic nespocitam}
     if (ErrorNumber=ErrOK)
        and(Express.Position<=length(Express.Expression))
        and(not(Express.Expression[Express.Position] in [';']))
        then Priority1(Result,Express);
     Evaluate := Result;
     ChangeANS(Result);
     if ErrorNumber<>ErrOK then IsValid := False
                               else IsValid := True;
end;

{****************************************************************************
 *                                    MAIN                                  *
 ****************************************************************************
 function initialisation - name MUST be in uppercase!!!}
begin
with FunctionData[1] do
begin;
{1}           Name:='COS'     ;Calculate:=EvalCos;
end;
with FunctionData[2] do
begin;
               Name:='SIN'     ;Calculate:=EvalSin;
end;
with FunctionData[3] do
begin;
               Name:='TG'      ;Calculate:=EvalTan;
end;
with FunctionData[4] do
begin;
               Name:='COTG'    ;Calculate:=EvalCoTan;
end;
with FunctionData[5] do
begin;
{5}            Name:='EXP'     ;Calculate:=EvalExp;
end;
with FunctionData[6] do
begin;
               Name:='LN'      ;Calculate:=EvalLn;
end;
with FunctionData[7] do
begin;
               Name:='LOG10'   ;Calculate:=EvalLog10;
end;
with FunctionData[8] do
begin;
               Name:='LOG2'    ;Calculate:=EvalLog2;
end;
with FunctionData[9] do
begin;
               Name:='LOG'     ;Calculate:=EvalLog;
end;
with FunctionData[10] do
begin;
{10}           Name:='ARCCOS'  ;Calculate:=EvalArcCos;
end;
with FunctionData[11] do
begin;
               Name:='ARCSIN'  ;Calculate:=EvalArcSin;
end;
with FunctionData[12] do
begin;
               Name:='ARCTG'   ;Calculate:=EvalArcTan;
end;
with FunctionData[13] do
begin;
               Name:='ARCCOTG' ;Calculate:=EvalArcCoTan;
end;
with FunctionData[14] do
begin;
               Name:='SQRT'    ;Calculate:=EvalSqrt;
end;
with FunctionData[15] do
begin;
{15}           Name:='SQR'     ;Calculate:=EvalSqr;
end;
with FunctionData[16] do
begin;
               Name:='RANDOM'  ;Calculate:=EvalRandom;
end;
with FunctionData[17] do
begin;
               Name:='TRUNC'   ;Calculate:=EvalTrunc;
end;
with FunctionData[18] do
begin;
               Name:='ROUND'   ;Calculate:=EvalRound;
end;
with FunctionData[19] do
begin;
               Name:='ABS'     ;Calculate:=EvalAbs;
end;
with FunctionData[20] do
begin;
{20}           Name:='POWER'   ;Calculate:=EvalPower;
end;
with FunctionData[21] do
begin;
               Name:='POWER2'  ;Calculate:=EvalPower2;
end;
with FunctionData[22] do
begin;
               Name:='POWER10' ;Calculate:=EvalPower10;
end;
with FunctionData[23] do
begin;
               Name:='FRAC'    ;Calculate:=EvalFrac;
end;
with FunctionData[24] do
begin;
               Name:='COSH'    ;Calculate:=EvalCosH;
end;
with FunctionData[25] do
begin;
{25}           Name:='SINH'    ;Calculate:=EvalSinH;
end;
with FunctionData[26] do
begin;
               Name:='TGH'     ;Calculate:=EvalTanH;
end;
with FunctionData[27] do
begin;
               Name:='COTGH'   ;Calculate:=EvalCoTanH;
end;
with FunctionData[28] do
begin;
               Name:='ARGCOSH' ;Calculate:=EvalArgCosH;
end;
with FunctionData[29] do
begin;
               Name:='ARGSINH' ;Calculate:=EvalArgSinH;
end;
with FunctionData[30] do
begin;
{30}           Name:='ARGTGH'  ;Calculate:=EvalArgTanH;
end;
with FunctionData[31] do
begin;
               Name:='ARGCOTGH';Calculate:=EvalArgCoTanH;
end;
with FunctionData[32] do
begin;
               Name:='COMB';Calculate:=EvalComb;
end;
with FunctionData[33] do
begin;
               Name:='VAR';Calculate:=EvalVar;
end;
with FunctionData[34] do
begin;
               Name:='RAD';Calculate:=EvalRad;
end;
with FunctionData[35] do
begin;
               Name:='DEG';Calculate:=EvalDeg;
end;
with FunctionData[36] do
begin;
               Name:='GRAD';Calculate:=EvalGrad;
end;
with FunctionData[37] do
begin;
               Name:='MAX';Calculate:=EvalMax;
end;
with FunctionData[38] do
begin;
               Name:='MIN';Calculate:=EvalMin;
end;
with FunctionData[39] do
begin;
               Name:='CU';Calculate:=EvalCub;
end;
with FunctionData[40] do
begin;
               Name:='CUR';Calculate:=EvalCubR;
end;
with FunctionData[41] do
begin;
               Name:='INV';Calculate:=EvalInv;
end;
with FunctionData[42] do
begin;
               Name:='MAXA';Calculate:=EvalMaxAbs;
end;
with FunctionData[43] do
begin;
               Name:='AVG';Calculate:=EvalAvg;
end;
with FunctionData[44] do
begin;
               Name:='GAV';Calculate:=EvalAvgG;
end;
with FunctionData[45] do
begin;
               Name:='HAV';Calculate:=EvalAvgH;
end;
with FunctionData[46] do
begin;
               Name:='MINA';Calculate:=EvalMinAbs;
end;
with FunctionData[47] do
begin;
               Name:='TAN'      ;Calculate:=EvalTan;
end;
with FunctionData[48] do
begin;
               Name:='COTAN'    ;Calculate:=EvalCoTan;
end;
with FunctionData[49] do
begin;
               Name:='ARCTAN'   ;Calculate:=EvalArcTan;
end;
with FunctionData[50] do
begin;
               Name:='ARCCOTAN' ;Calculate:=EvalArcCoTan;
end;
with FunctionData[51] do
begin;
               Name:='TGH'     ;Calculate:=EvalTanH;
end;
with FunctionData[52] do
begin;
               Name:='COTANH'   ;Calculate:=EvalCoTanH;
end;
with FunctionData[53] do
begin;
               Name:='ARGTANH'  ;Calculate:=EvalArgTanH;
end;
with FunctionData[54] do
begin;
               Name:='ARGCOTANH';Calculate:=EvalArgCoTanH;
end;
with FunctionData[55] do
begin;
               Name:='CEIL';Calculate:=EvalCeil;
end;
with FunctionData[56] do
begin;
               Name:='DERIVATE';Calculate:=EvalDerivate;
end;
with FunctionData[57] do
begin;
               Name:='SOLVE';Calculate:=EvalSolve;
end;
with FunctionData[58] do
begin;
               Name:='SOLVEG';Calculate:=EvalSolveGuess;
end;
with FunctionData[59] do
begin;
               Name:='SUM';Calculate:=EvalSum;
end;
with FunctionData[60] do
begin;
               Name:='INTG';Calculate:=EvalIntg;
end;
with FunctionData[61] do
begin;
               Name:='INTGS';Calculate:=EvalIntgS;
end;
with FunctionData[62] do
begin;
               Name:='DERIVATE2';Calculate:=EvalDerivate2;
end;
with FunctionData[63] do
begin;
               Name:='SIGN';Calculate:=EvalSign;
end;
with FunctionData[64] do
begin;
               Name:='SGN';Calculate:=EvalSign;
end;
with FunctionData[65] do
begin;
               Name:='SUMTABLE';Calculate:=EvalSumTable;
end;
end.
